from typing import *
from pathlib import Path

dis = [f'DI{i}' for i in range(1, 9)] + ['DIAUX']
dos = [f'DO{i}' for i in range(1, 9)] + ['DOAUX']
ios = dis + dos

parameters = [
    # Type, name template, function name (will be surrounded by get and Praram)
    ('io_mode_t', 'ParamAPP_{}_MODE', 'Mode', ios),
    ('uint8_t', 'ParamAPP_{}_DEBOUNCE', 'Debounce', ios),
    ('init_state_t', 'ParamAPP_{}_INIT_STATE', 'InitState', ios),
    ('switch_type_t', 'ParamAPP_{}_SWITCH_TYPE', 'SwitchType', ios),
    ('uint16_t', 'ParamAPP_{}_EM_PULSES', 'EMPulses', ios),
    ('em_cyclic_t', 'ParamAPP_{}_EM_CYCLIC', 'EMCyclic', ios),
    ('uint16_t', 'ParamAPP_{}_CB_KWH', 'EMCbKwm', ios),
    ('uint16_t', 'ParamAPP_{}_CYCLIC_DELAY', 'EMCyclicDelay', ios),
    ('uint8_t', 'ParamAPP_{}_INIT_STATE_OUT', 'InitStateOut', ios),
    ('uint8_t', 'ParamAPP_{}_INIT_STATE_PWM_OUT', 'InitStatePwmOut', ios),
    ('uint16_t', 'ParamAPP_{}_LP_DELAY', 'LpDelay', ios),
    ('uint8_t', 'ParamAPP_{}_IN_DC_ENABLED', 'DcEnabled', ios),
    ('uint8_t', 'ParamAPP_{}_IN_LP_ENABLED', 'LpEnabled', ios),

    ('scene_type_t', 'ParamAPP_{}_SCENE_TYPE_A', 'SceneTypeA', dis),
    ('scene_type_t', 'ParamAPP_{}_SCENE_TYPE_B', 'SceneTypeB', dis),
    ('scene_type_t', 'ParamAPP_{}_SCENE_TYPE_C', 'SceneTypeC', dis),
    ('scene_type_t', 'ParamAPP_{}_SCENE_TYPE_D', 'SceneTypeD', dis),

    ('uint8_t', 'ParamAPP_{}_SCENE_TYPE_A_1BIT_VALUE', 'SceneValue1BitA', dis),
    ('uint8_t', 'ParamAPP_{}_SCENE_TYPE_B_1BIT_VALUE', 'SceneValue1BitB', dis),
    ('uint8_t', 'ParamAPP_{}_SCENE_TYPE_C_1BIT_VALUE', 'SceneValue1BitC', dis),
    ('uint8_t', 'ParamAPP_{}_SCENE_TYPE_D_1BIT_VALUE', 'SceneValue1BitD', dis),

    ('uint8_t', 'ParamAPP_{}_SCENE_TYPE_A_8BIT_VALUE', 'SceneValue8BitA', dis),
    ('uint8_t', 'ParamAPP_{}_SCENE_TYPE_B_8BIT_VALUE', 'SceneValue8BitB', dis),
    ('uint8_t', 'ParamAPP_{}_SCENE_TYPE_C_8BIT_VALUE', 'SceneValue8BitC', dis),
    ('uint8_t', 'ParamAPP_{}_SCENE_TYPE_D_8BIT_VALUE', 'SceneValue8BitD', dis),
]

com_objects = [
    # Com object function template, index name template, function name same as params
    ('KoAPP_{}_IN', 'APP_Ko{}_IN', 'In', ios),
    ('KoAPP_{}_IN_DC', 'APP_Ko{}_IN_DC', 'InDc', ios),
    ('KoAPP_{}_IN_LP', 'APP_Ko{}_IN_LP', 'InLp', ios),
    ('KoAPP_{}_IN_EM', 'APP_Ko{}_IN_EM', 'InEm', ios),
    ('KoAPP_{}_OUT', 'APP_Ko{}_OUT', 'Out', ios),
    ('KoAPP_{}_OUT_PWM', 'APP_Ko{}_OUT_PWM', 'OutPwm', ios),

    ('KoAPP_{}_SCENE_A', 'APP_Ko{}_SCENE_A', 'SceneA', dis),
    ('KoAPP_{}_SCENE_B', 'APP_Ko{}_SCENE_B', 'SceneB', dis),
    ('KoAPP_{}_SCENE_C', 'APP_Ko{}_SCENE_C', 'SceneC', dis),
    ('KoAPP_{}_SCENE_D', 'APP_Ko{}_SCENE_D', 'SceneD', dis),

    ('KoAPP_{}_SCENE_A_8BIT', 'APP_Ko{}_SCENE_A_8BIT', 'SceneA8Bit', dis),
    ('KoAPP_{}_SCENE_B_8BIT', 'APP_Ko{}_SCENE_B_8BIT', 'SceneB8Bit', dis),
    ('KoAPP_{}_SCENE_C_8BIT', 'APP_Ko{}_SCENE_C_8BIT', 'SceneC8Bit', dis),
    ('KoAPP_{}_SCENE_D_8BIT', 'APP_Ko{}_SCENE_D_8BIT', 'SceneD8Bit', dis),
]


base_path = Path(__file__).parent
generated_path = base_path / 'include' / 'generated.h'


def parameter_get_function(param_type: str, param_def_template: str, function_name: str, ios: List[str]) -> str:
    l = list()
    l.append(f'{param_type} get{function_name}Param(uint8_t index)')
    l.append('{')

    if_or_elif = 'if'
    for index, io_name in enumerate(ios):
        if index != len(ios) - 1:
            l.append(f'{if_or_elif} (index == {index})')
            l.append('{')
        else:
            l.append('/* Default return */')

        l.append(f'return static_cast<{param_type}>'
                 f'({param_def_template.format(io_name)});')

        if index != len(ios) - 1:
            l.append('}')
        if_or_elif = 'else if'

    l.append('}')
    return '\n'.join(l) + ('\n' * 2)


def com_object_get_function(com_object_def_template: str, function_name: str, ios: List[str]) -> str:
    l = list()
    l.append(f'GroupObject &getGroupObject{function_name}(uint8_t index)')
    l.append('{')

    if_or_elif = 'if'
    for index, io_name in enumerate(ios):
        if index != len(ios) - 1:
            l.append(f'{if_or_elif} (index == {index})')
            l.append('{')
        else:
            l.append('/* Default return */')

        l.append(f'return {com_object_def_template.format(io_name)};')

        if index != len(ios) - 1:
            l.append('}')
        if_or_elif = 'else if'

    l.append('}')
    return '\n'.join(l) + ('\n' * 2)


def index_get_function(com_object_def_number_template: str,  function_name: str, ios: List[str]) -> str:
    l = list()
    l.append(f'uint8_t getIndex{function_name}(GroupObject &go)')
    l.append('{')

    if_or_elif = 'if'
    for index, io_name in enumerate(ios):
        if index != len(ios) - 1:
            l.append(f'{if_or_elif}'
                     f'(go.asap() == {com_object_def_number_template.format(io_name)})')
            l.append('{')
        else:
            l.append('/* Default return */')

        l.append(f'return {index};')

        if index != len(ios) - 1:
            l.append('}')
        if_or_elif = 'else if'

    l.append('}')
    return '\n'.join(l) + ('\n' * 2)


generated = """
#pragma once
#include <Arduino.h>
#include <knx.h>

#include "knx_param_types.h"
#include "knxprod_AF01_10.h"

/*
    This file is generated by "generate.py" in root of project.
    Getting com objects could be done based on pin number and com object number.
    But since params are ordered automatically to save some space, getters are easier way to get them.
*/

""".lstrip()

for parameter in parameters:
    generated += parameter_get_function(*parameter)

for com_object_def_template, com_object_def_number_template, function_name, ios in com_objects:
    generated += com_object_get_function(
        com_object_def_template,
        function_name,
        ios
    )
    generated += index_get_function(
        com_object_def_number_template,
        function_name,
        ios
    )

generated_path.write_text(generated)
