#pragma once
#include <Arduino.h>
#include <knx.h>

#include "board_io.h"
#include "board_ios.h"
#include "knx_utils.h"

void actionSwitch(KnxBoardIO *io, bool state)
{
    setNthByte(&io->value, 0, static_cast<uint8_t>(state));

    getGroupObjectIn(io->getIndex()).value(getNthByte(io->value, 0));
}

void actionButton(KnxBoardIO *io)
{
    bool temp = static_cast<bool>(getNthByte(io->value, 0));
    setNthByte(&io->value, 0, static_cast<uint8_t>(!temp));

    getGroupObjectIn(io->getIndex()).value(getNthByte(io->value, 0));
}

void actionButtonDc(KnxBoardIO *io)
{
    bool temp = static_cast<bool>(getNthByte(io->value, 1));
    setNthByte(&io->value, 1, static_cast<uint8_t>(!temp));

    getGroupObjectInDc(io->getIndex()).value(getNthByte(io->value, 1));
}

void actionButtonLp(KnxBoardIO *io)
{
    bool temp = static_cast<bool>(getNthByte(io->value, 2));
    setNthByte(&io->value, 2, static_cast<uint8_t>(!temp));

    getGroupObjectInLp(io->getIndex()).value(getNthByte(io->value, 2));
}

void actionEnergyMeter(KnxBoardIO *io, float_t energy)
{
    getGroupObjectInEm(io->getIndex()).value(static_cast<int32_t>(energy));
}

void actionRGB(GroupObject &go)
{
    KnxRgbIos rgbIos = getRgbIos();

    uint32_t color = static_cast<uint32_t>(go.value());

    uint8_t r = getNthByte(color, 2);
    uint8_t g = getNthByte(color, 1);
    uint8_t b = getNthByte(color, 0);

    setNthByte(&rgbIos.rIo->value, 0, r);
    setNthByte(&rgbIos.gIo->value, 0, g);
    setNthByte(&rgbIos.bIo->value, 0, b);

    if (getNthByte(rgbIos.rIo->value, 1))
    {
        rgbIos.rIo->writeAnalog(r);
        rgbIos.gIo->writeAnalog(g);
        rgbIos.bIo->writeAnalog(b);
    }
    else
    {
        rgbIos.rIo->writeAnalog(0);
        rgbIos.gIo->writeAnalog(0);
        rgbIos.bIo->writeAnalog(0);
    }
}

void actionRGBOnOff(GroupObject &go)
{
    KnxRgbIos rgbIos = getRgbIos();

    uint8_t value = static_cast<uint8_t>(go.value()) & 1;

    setNthByte(&rgbIos.rIo->value, 1, value);

    if (value)
    {
        uint8_t r = getNthByte(rgbIos.rIo->value, 0);
        uint8_t g = getNthByte(rgbIos.gIo->value, 0);
        uint8_t b = getNthByte(rgbIos.bIo->value, 0);

        rgbIos.rIo->writeAnalog(r);
        rgbIos.gIo->writeAnalog(g);
        rgbIos.bIo->writeAnalog(b);
    }
    else
    {
        rgbIos.rIo->writeAnalog(0);
        rgbIos.gIo->writeAnalog(0);
        rgbIos.bIo->writeAnalog(0);
    }
}

void actionOut(GroupObject &go)
{
    KnxBoardIO *io = getIo(getIndexOut(go));

    io->value = static_cast<uint8_t>(go.value()) & 1;

    io->writeDigital(static_cast<bool>(io->value));
}

void actionOutPwm(GroupObject &go)
{
    KnxBoardIO *io = getIo(getIndexOutPwm(go));

    setNthByte(&io->value, 0, static_cast<uint8_t>(go.value()));

    io->writeAnalog(getNthByte(io->value, 1) ? getNthByte(io->value, 0) : 0);
}

void actionOutOnOffPwm(GroupObject &go)
{
    KnxBoardIO *io = getIo(getIndexOut(go));

    setNthByte(&io->value, 1, static_cast<uint8_t>(go.value()));

    io->writeAnalog(getNthByte(io->value, 1) ? getNthByte(io->value, 0) : 0);
}

/*
    Scenes On/Off
*/
void actionSceneIn(KnxBoardIO *io)
{
    for (uint8_t i = 0; i < 4; i++)
    {
        io->setShouldUpdateRecallValue(i, false);

        uint8_t value = io->getSceneEnabled() ? getNthByte(io->value, i) : getNthByte(*io->getRecallValuePtr(), i);

        if (io->getSceneType(i) == SCENE_1BIT)
        {
            getGroupObjectScene1Bit(io->getIndex(), i).value(value);
        }
        else
        {
            getGroupObjectScene8Bit(io->getIndex(), i).value(value);
        }
    }

    io->toggleSceneEnabled();
}

/*
    1Bit scenes
*/
void actionSceneA(GroupObject &go)
{
    KnxBoardIO *io = getIo(getIndexSceneA(go));

    if (io->shouldUpdateRecallValue(0))
    {
        setNthByte(io->getRecallValuePtr(), 0, static_cast<uint8_t>(go.value()));
    }
    io->setShouldUpdateRecallValue(0, true);
}

void actionSceneB(GroupObject &go)
{
    KnxBoardIO *io = getIo(getIndexSceneB(go));

    if (io->shouldUpdateRecallValue(1))
    {
        setNthByte(io->getRecallValuePtr(), 1, static_cast<uint8_t>(go.value()));
    }
    io->setShouldUpdateRecallValue(1, true);
}

void actionSceneC(GroupObject &go)
{
    KnxBoardIO *io = getIo(getIndexSceneC(go));

    if (io->shouldUpdateRecallValue(2))
    {
        setNthByte(io->getRecallValuePtr(), 2, static_cast<uint8_t>(go.value()));
    }
    io->setShouldUpdateRecallValue(2, true);
}

void actionSceneD(GroupObject &go)
{
    KnxBoardIO *io = getIo(getIndexSceneD(go));

    if (io->shouldUpdateRecallValue(3))
    {
        setNthByte(io->getRecallValuePtr(), 3, static_cast<uint8_t>(go.value()));
    }
    io->setShouldUpdateRecallValue(3, true);
}

/*
    8Bit scenes
*/
void actionScene8BitA(GroupObject &go)
{
    KnxBoardIO *io = getIo(getIndexSceneA8Bit(go));

    if (io->shouldUpdateRecallValue(0))
    {
        setNthByte(io->getRecallValuePtr(), 0, static_cast<uint8_t>(go.value()));
    }
    io->setShouldUpdateRecallValue(0, true);
}

void actionScene8BitB(GroupObject &go)
{
    KnxBoardIO *io = getIo(getIndexSceneB8Bit(go));

    if (io->shouldUpdateRecallValue(1))
    {
        setNthByte(io->getRecallValuePtr(), 1, static_cast<uint8_t>(go.value()));
    }
    io->setShouldUpdateRecallValue(1, true);
}

void actionScene8BitC(GroupObject &go)
{
    KnxBoardIO *io = getIo(getIndexSceneC8Bit(go));

    if (io->shouldUpdateRecallValue(2))
    {
        setNthByte(io->getRecallValuePtr(), 2, static_cast<uint8_t>(go.value()));
    }
    io->setShouldUpdateRecallValue(2, true);
}

void actionScene8BitD(GroupObject &go)
{
    KnxBoardIO *io = getIo(getIndexSceneD8Bit(go));

    if (io->shouldUpdateRecallValue(3))
    {
        setNthByte(io->getRecallValuePtr(), 3, static_cast<uint8_t>(go.value()));
    }
    io->setShouldUpdateRecallValue(3, true);
}
