#pragma once
#include <Arduino.h>

#include "board_io.h"
#include "knx_params_cos.h"

// Get input initial state based on state param
bool inputInitialValue(KnxBoardIO *io, init_state_t state)
{
    bool result = false;
    switch (state)
    {
    case STATE_ON:
    case STATE_OFF:
        result = state == STATE_ON;
        break;
    case STATE_READ:
        result = io->read();
        break;
    case STATE_READ_INVERTED:
        result = !io->read();
        break;
    default:
        break;
    }
    return result;
}

// Get nth byte of number
uint8_t getNthByte(uint32_t number, uint8_t n)
{
    /*
    We use a bitwise right shift operation (>>) to shift the byte of interest to the least significant position,
    and then use a bitwise AND operation (&) with a mask (0xff) to extract only the least significant byte.
    */
    uint8_t nth = n * 8;
    return (number >> nth) & 0xff;
}

// Set nth byte of number
void setNthByte(uint32_t *number, uint8_t n, uint8_t value)
{
    /*
    We use a bitwise AND operation with a mask to clear the byte at position n in the number variable.
    Then, we use a bitwise OR operation to set the byte at position n to the desired value.
    The nth is calculated as n * 8 since each byte consists of 8 bits in a 32-bit integer.
    */
    uint8_t nth = n * 8;
    *number &= ~(0xFF << nth);
    *number |= (value << nth);
}
