import json
import itertools
from typing import *
from pathlib import Path
from pprint import pprint

from models.uid_counter import uid_counter as uid

from models.parameter import Parameter
from models.com_object import ComObject
from models.parameter_type import ParameterType, ParameterTypes, ParameterTypeEnum
from models.dynamics.main import DynMain
from models.dynamics.channel_independent import DynChannelIndependent
from models.dynamics.channel import DynChannel
from models.dynamics.block import DynBlock
from models.dynamics.separator import DynSeparator, DynSeparatorHint
from models.dynamics.parameter import DynParameter
from models.dynamics.choose import DynChoose
from models.dynamics.when import DynWhen
from models.dynamics.com_object import DynComObject

dis = [f'DI{i}' for i in range(1, 9)] + ['DIAUX']
dos = [f'DO{i}' for i in range(1, 9)] + ['DOAUX']
ios = dis + dos
rgb_ios = [ios[i] for i in [14, 15, 16]]

param_types = list()
parameters = list()
com_objects = list()

# Param types
rgp_pt = ParameterType.from_args(uid(ParameterType), 'RGB', ParameterTypes.NUMBER_UINT, 0, 1).with_hint('CheckBox')
mode_pt = ParameterType.from_args(uid(ParameterType), 'IoMode', ParameterTypes.ENUM).with_enums(
    ParameterTypeEnum.from_args('MODE_NONE', uid('IoMode'), 'Disabled'),
    ParameterTypeEnum.from_args('MODE_IN_SWITCH', uid('IoMode'), 'Switch input'),
    ParameterTypeEnum.from_args('MODE_IN_BUTTON', uid('IoMode'), 'Button input'),
    ParameterTypeEnum.from_args('MODE_IN_BUTTON_DC', uid('IoMode'), 'Button input double click'),
    ParameterTypeEnum.from_args('MODE_IN_BUTTON_LP', uid('IoMode'), 'Button input long press'),
    ParameterTypeEnum.from_args('MODE_IN_ENERGY_METER', uid('IoMode'), 'Energy meter input'),
    ParameterTypeEnum.from_args('MODE_OUT', uid('IoMode'), 'Switch output'),
    ParameterTypeEnum.from_args('MODE_OUT_PWM', uid('IoMode'), 'PWM output')
)
mode_full_pt = ParameterType.from_args(uid(ParameterType), 'IoModeFull', ParameterTypes.ENUM).with_enums(
    ParameterTypeEnum.from_args('MODE_NONE', uid('IoModeFull'), 'Disabled'),
    ParameterTypeEnum.from_args('MODE_IN_SWITCH', uid('IoModeFull'), 'Switch input'),
    ParameterTypeEnum.from_args('MODE_IN_BUTTON', uid('IoModeFull'), 'Button input'),
    ParameterTypeEnum.from_args('MODE_IN_BUTTON_DC', uid('IoModeFull'), 'Button input double click'),
    ParameterTypeEnum.from_args('MODE_IN_BUTTON_LP', uid('IoModeFull'), 'Button input long press'),
    ParameterTypeEnum.from_args('MODE_IN_ENERGY_METER', uid('IoModeFull'), 'Energy meter input'),
    ParameterTypeEnum.from_args('MODE_OUT', uid('IoModeFull'), 'Switch output'),
    ParameterTypeEnum.from_args('MODE_OUT_PWM', uid('IoModeFull'), 'PWM output'),
    ParameterTypeEnum.from_args('MODE_SCENES', uid('IoModeFull'), 'Scene control')
)
debounce_pt = ParameterType.from_args(uid(ParameterType), 'Debounce', ParameterTypes.NUMBER_UINT, 20, 100, 5)
lp_delay_pt = ParameterType.from_args(uid(ParameterType), 'LongPressDelay', ParameterTypes.NUMBER_UINT,
                                      500, 2500, 100)
init_state_pt = ParameterType.from_args(uid(ParameterType), 'InitState', ParameterTypes.ENUM).with_enums(
    ParameterTypeEnum.from_args('OFF', uid('InitState'), 'Low'),
    ParameterTypeEnum.from_args('ON', uid('InitState'), 'High'),
    ParameterTypeEnum.from_args('READ', uid('InitState'), 'Read current state of input'),
    ParameterTypeEnum.from_args('READ_INVERTED', uid('InitState'), 'Read current state of input and invert it'),
)
switch_type_pt = ParameterType.from_args(uid(ParameterType), 'SwitchType', ParameterTypes.ENUM).with_enums(
    ParameterTypeEnum.from_args('NORMALLY_OPEN', uid('SwitchType'), 'Normally open'),
    ParameterTypeEnum.from_args('NORMALLY_CLOSED', uid('SwitchType'), 'Normally closed'),
)
em_pulses_pt = ParameterType.from_args(uid(ParameterType), 'EMPulses', ParameterTypes.NUMBER_UINT, 10, 10000, 100)
em_cyclic_pt = ParameterType.from_args(uid(ParameterType), 'EMCyclic', ParameterTypes.ENUM).with_enums(
    ParameterTypeEnum.from_args('SEND_PER_KWH', uid('EMCyclic'), 'Send per kWh'),
    ParameterTypeEnum.from_args('SEND_CYCLIC', uid('EMCyclic'), 'Send cyclically'),
)
cb_kwh_pt = ParameterType.from_args(uid(ParameterType), 'EMCbKwh', ParameterTypes.NUMBER_UINT, 1, 100, 1)
cyclic_delay_pt = ParameterType.from_args(uid(ParameterType), 'EMCyclicDelay', ParameterTypes.NUMBER_UINT, 500,
                                          60000, 100)
in_dc_enabled_pt = ParameterType.from_args(uid(ParameterType), 'InDcEnabled', ParameterTypes.NUMBER_UINT, 0, 1
                                           ).with_hint('CheckBox')
in_lp_enabled_pt = ParameterType.from_args(uid(ParameterType), 'InLpEnabled', ParameterTypes.NUMBER_UINT, 0, 1
                                           ).with_hint('CheckBox')
out_state_pt = ParameterType.from_args(uid(ParameterType), 'InitStateOut', ParameterTypes.ENUM).with_enums(
    ParameterTypeEnum.from_args('LOW', uid('InitStateOut'), 'Low'),
    ParameterTypeEnum.from_args('HIGH', uid('InitStateOut'), 'High')
)
out_state_pwm_pt = ParameterType.from_args(uid(ParameterType), 'InitStatePwmOut', ParameterTypes.NUMBER_UINT,
                                           0, 255, 1)
scene_type_pt = ParameterType.from_args(uid(ParameterType), 'SceneMode', ParameterTypes.ENUM).with_enums(
    ParameterTypeEnum.from_args('1BIT', uid('SceneMode'), '1 Bit mode'),
    ParameterTypeEnum.from_args('8BIT', uid('SceneMode'), '8 Bit mode'),
)
scene_1b_value_pt = ParameterType.from_args(uid(ParameterType), 'Scene1bValue', ParameterTypes.ENUM).with_enums(
    ParameterTypeEnum.from_args('ON', uid('Scene1bValue'), 'On'),
    ParameterTypeEnum.from_args('OFF', uid('Scene1bValue'), 'Off')
)
scene_8b_value_pt = ParameterType.from_args(uid(ParameterType), 'Scene8bValue', ParameterTypes.NUMBER_UINT,
                                            0, 255, 1)

param_types.extend([
    rgp_pt, mode_pt, mode_full_pt, debounce_pt, init_state_pt, em_cyclic_pt, em_pulses_pt, cb_kwh_pt, cyclic_delay_pt,
    out_state_pt, out_state_pwm_pt, lp_delay_pt, switch_type_pt, in_dc_enabled_pt, in_lp_enabled_pt,
    scene_type_pt, scene_1b_value_pt, scene_8b_value_pt
])

# Board params
rgb_enabled_param = Parameter.from_args(uid(Parameter), 'RGB_ENABLED', rgp_pt, '0', 'Enable RGB')
parameters.append(rgb_enabled_param)

# Board com objects
input_flags = [True, False, True, True, False, False]
output_flags = [True, True, True, False, False, False]
scene_flags = [True, True, True, True, False, True]

rgb_co = ComObject.from_args(uid(ComObject), 'RGB', '232', '600').with_flags(*input_flags)
rgb_on_off_co = ComObject.from_args(uid(ComObject), 'RGB_ON_OFF', '1', '1').with_flags(*input_flags)
com_objects.extend([rgb_co, rgb_on_off_co])

# Board dynamics
independent_channel = DynChannelIndependent().extend_items(DynBlock.from_args('Board').extend_items(
    DynSeparator.from_args('H8I8O', DynSeparatorHint.HEADLINE),
    DynSeparator.from_args(
        'RGB channel info',
        DynSeparatorHint.INFO,
        f'If enabled channels {", ".join(rgb_ios)} '
        f'will be dedicated from RGB PWM.'
    ),
    DynParameter.from_parameter(rgb_enabled_param),
    DynChoose.from_param('RGB CO', rgb_enabled_param).extend_items(
        DynWhen.from_condition('1').extend_items(
            DynComObject.from_com_object(rgb_co),
            DynComObject.from_com_object(rgb_on_off_co)
        )
    )
))
di_channel = DynChannel.from_args(uid(DynChannel), 'DI')
do_channel = DynChannel.from_args(uid(DynChannel), 'DO')
dynamics = DynMain().extend_items(independent_channel, di_channel, do_channel)

for io_index, io_name in enumerate(ios):
    # Params
    io_mode_param = Parameter.from_args(uid(Parameter), f'{io_name}_MODE',
                                        mode_full_pt if io_index < len(dis) else mode_pt,
                                        0, f'{io_name} function')
    debounce_param = Parameter.from_args(uid(Parameter), f'{io_name}_DEBOUNCE', debounce_pt, 20,
                                         f'{io_name} debounce', 'ms')
    long_press_delay_param = Parameter.from_args(uid(Parameter), f'{io_name}_LP_DELAY', lp_delay_pt, 1500,
                                                 f'{io_name} long press delay', 'ms')
    init_state_param = Parameter.from_args(uid(Parameter), f'{io_name}_INIT_STATE', init_state_pt, 0,
                                           f'{io_name} init state')
    switch_type_param = Parameter.from_args(uid(Parameter), f'{io_name}_SWITCH_TYPE', switch_type_pt, 0,
                                            f'{io_name} switch type')
    em_pulses_param = Parameter.from_args(uid(Parameter), f'{io_name}_EM_PULSES', em_pulses_pt, 1600,
                                          f'{io_name} energy meter pulses per kWh', 'pulses')
    em_cyclic_param = Parameter.from_args(uid(Parameter), f'{io_name}_EM_CYCLIC', em_cyclic_pt, 0,
                                          f'{io_name} send cyclically or per kWh.')
    em_cb_kwh_param = Parameter.from_args(uid(Parameter), f'{io_name}_CB_KWH', cb_kwh_pt, 1,
                                          f'{io_name} kWh count', 'kWh')
    em_cyclic_delay_param = Parameter.from_args(uid(Parameter), f'{io_name}_CYCLIC_DELAY', cyclic_delay_pt, 1000,
                                                f'{io_name} cycle delay', 'ms')
    out_state_param = Parameter.from_args(uid(Parameter), f'{io_name}_INIT_STATE_OUT', out_state_pt, 0,
                                          f'{io_name} init state')
    out_state_pwm_param = Parameter.from_args(uid(Parameter), f'{io_name}_INIT_STATE_PWM_OUT', out_state_pwm_pt, 0,
                                              f'{io_name} init state PWM')
    in_dc_enabled = Parameter.from_args(uid(Parameter), f'{io_name}_IN_DC_ENABLED', in_dc_enabled_pt, 0,
                                        f'{io_name} enable alternative double click action')
    in_lp_enabled = Parameter.from_args(uid(Parameter), f'{io_name}_IN_LP_ENABLED', in_lp_enabled_pt, 0,
                                        f'{io_name} enable alternative long press action')

    parameters.extend([io_mode_param, debounce_param, init_state_param, switch_type_param,
                       em_pulses_param, em_cyclic_param, em_cb_kwh_param, em_cyclic_delay_param,
                       out_state_param, out_state_pwm_param, long_press_delay_param, in_dc_enabled,
                       in_lp_enabled])

    # Com objects
    in_co = ComObject.from_args(uid(ComObject), f'{io_name}_IN', '1', '1').with_flags(*input_flags)
    in_dc_co = ComObject.from_args(uid(ComObject), f'{io_name}_IN_DC', '1', '1').with_flags(*input_flags)
    in_lp_co = ComObject.from_args(uid(ComObject), f'{io_name}_IN_LP', '1', '1').with_flags(*input_flags)
    in_em_co = ComObject.from_args(uid(ComObject), f'{io_name}_IN_EM', '13', '13').with_flags(*input_flags)
    out_co = ComObject.from_args(uid(ComObject), f'{io_name}_OUT', '1', '1').with_flags(*output_flags)
    out_pwm_co = ComObject.from_args(uid(ComObject), f'{io_name}_OUT_PWM', '5', '1').with_flags(*output_flags)

    com_objects.extend([in_co, in_dc_co, in_lp_co, in_em_co, out_co, out_pwm_co])

    # Scenes
    scene_dynamics = [None] * len(ios)
    if io_index < len(dis):
        # Scene params
        scene_type_params = [
            Parameter.from_args(uid(Parameter), f'{io_name}_SCENE_TYPE_A', scene_type_pt, 0),
            Parameter.from_args(uid(Parameter), f'{io_name}_SCENE_TYPE_B', scene_type_pt, 0),
            Parameter.from_args(uid(Parameter), f'{io_name}_SCENE_TYPE_C', scene_type_pt, 0),
            Parameter.from_args(uid(Parameter), f'{io_name}_SCENE_TYPE_D', scene_type_pt, 0),
        ]
        scene_1b_value_params = [
            Parameter.from_args(uid(Parameter), f'{io_name}_SCENE_TYPE_A_1BIT_VALUE', scene_1b_value_pt, 0),
            Parameter.from_args(uid(Parameter), f'{io_name}_SCENE_TYPE_B_1BIT_VALUE', scene_1b_value_pt, 0),
            Parameter.from_args(uid(Parameter), f'{io_name}_SCENE_TYPE_C_1BIT_VALUE', scene_1b_value_pt, 0),
            Parameter.from_args(uid(Parameter), f'{io_name}_SCENE_TYPE_D_1BIT_VALUE', scene_1b_value_pt, 0),
        ]
        scene_8b_value_params = [
            Parameter.from_args(uid(Parameter), f'{io_name}_SCENE_TYPE_A_8BIT_VALUE', scene_8b_value_pt, 0),
            Parameter.from_args(uid(Parameter), f'{io_name}_SCENE_TYPE_B_8BIT_VALUE', scene_8b_value_pt, 0),
            Parameter.from_args(uid(Parameter), f'{io_name}_SCENE_TYPE_C_8BIT_VALUE', scene_8b_value_pt, 0),
            Parameter.from_args(uid(Parameter), f'{io_name}_SCENE_TYPE_D_8BIT_VALUE', scene_8b_value_pt, 0),
        ]
        parameters.extend(scene_type_params + scene_1b_value_params + scene_8b_value_params)

        # Scene cos
        scenes_1b_cos = [
            ComObject.from_args(uid(ComObject), f'{io_name}_SCENE_A', '1', '1').with_flags(*scene_flags),
            ComObject.from_args(uid(ComObject), f'{io_name}_SCENE_B', '1', '1').with_flags(*scene_flags),
            ComObject.from_args(uid(ComObject), f'{io_name}_SCENE_C', '1', '1').with_flags(*scene_flags),
            ComObject.from_args(uid(ComObject), f'{io_name}_SCENE_D', '1', '1').with_flags(*scene_flags)
        ]
        scenes_8b_cos = [
            ComObject.from_args(uid(ComObject), f'{io_name}_SCENE_A_8BIT', '5', '1').with_flags(*scene_flags),
            ComObject.from_args(uid(ComObject), f'{io_name}_SCENE_B_8BIT', '5', '1').with_flags(*scene_flags),
            ComObject.from_args(uid(ComObject), f'{io_name}_SCENE_C_8BIT', '5', '1').with_flags(*scene_flags),
            ComObject.from_args(uid(ComObject), f'{io_name}_SCENE_D_8BIT', '5', '1').with_flags(*scene_flags)
        ]
        com_objects.extend(scenes_1b_cos + scenes_8b_cos)

        # scene dynamics
        scene_dynamics[io_index] = DynWhen.from_condition('8').extend_items(
            DynParameter.from_parameter(debounce_param),
            DynParameter.from_parameter(switch_type_param),
            DynSeparator.from_args('Scenes separator', DynSeparatorHint.RULER),

            *itertools.chain.from_iterable([
                (
                    DynParameter.from_parameter(param),
                    DynChoose.from_param(f'Scene mode {param.name}', param).extend_items(
                        DynWhen.from_condition('0').extend_items(
                            DynComObject.from_com_object(scenes_1b_cos[i]),
                            DynParameter.from_parameter(scene_1b_value_params[i])
                        ),
                        DynWhen.from_condition('1').extend_items(
                            DynComObject.from_com_object(scenes_8b_cos[i]),
                            DynParameter.from_parameter(scene_8b_value_params[i])
                        )
                    )
                )
                for i, param in enumerate(scene_type_params)
            ])
        )

    # Dynamics
    channel = di_channel if io_name.startswith('DI') else do_channel
    if io_name in rgb_ios:  # If RGB enabled, don't display RGB pins
        channel_rgb = DynWhen.from_condition('0')
        channel.extend_items(DynChoose.from_param('RGB_ENABLED', rgb_enabled_param).with_items(channel_rgb))
        channel = channel_rgb

    channel.extend_items(DynBlock.from_args(io_name).extend_items(
        DynParameter.from_parameter(io_mode_param),
        DynChoose.from_param('Mode', io_mode_param).extend_items(
            DynWhen.from_condition('1 2').extend_items(
                DynComObject.from_com_object(in_co)
            ),
            DynWhen.from_condition('3').extend_items(
                DynComObject.from_com_object(in_dc_co)
            ),
            DynWhen.from_condition('4').extend_items(
                DynComObject.from_com_object(in_lp_co),
                DynParameter.from_parameter(long_press_delay_param)
            ),
            DynWhen.from_condition('5').extend_items(
                DynComObject.from_com_object(in_em_co),
                DynParameter.from_parameter(em_pulses_param),
                DynParameter.from_parameter(em_cyclic_param),
                DynChoose.from_param('Cyclic', em_cyclic_param).extend_items(
                    DynWhen.from_condition('0').extend_items(
                        DynParameter.from_parameter(em_cb_kwh_param)
                    ),
                    DynWhen.from_condition('1').extend_items(
                        DynParameter.from_parameter(em_cyclic_delay_param)
                    )
                )
            ),
            DynWhen.from_condition('6 7').extend_items(
                DynComObject.from_com_object(out_co),
                DynParameter.from_parameter(out_state_param)
            ),
            DynWhen.from_condition('7').extend_items(
                DynComObject.from_com_object(out_pwm_co),
                DynParameter.from_parameter(out_state_pwm_param)
            ),
            scene_dynamics[io_index],
            DynWhen.from_condition('1 2 3 4 5').extend_items(
                DynParameter.from_parameter(debounce_param),
                DynParameter.from_parameter(init_state_param),
                DynParameter.from_parameter(switch_type_param),
            ),
            DynWhen.from_condition('2').extend_items(
                DynSeparator.from_args(
                    'Double click enabled',
                    DynSeparatorHint.INFO,
                    'Enable alternative double click action?'
                ),
                DynParameter.from_parameter(in_dc_enabled),
                DynChoose.from_param('Double click enabled', in_dc_enabled).extend_items(
                    DynWhen.from_condition('1').extend_items(
                        DynComObject.from_com_object(in_dc_co)
                    )
                ),
                DynSeparator.from_args(
                    'Long press enabled',
                    DynSeparatorHint.INFO,
                    'Enable alternative long press action?'
                ),
                DynParameter.from_parameter(in_lp_enabled),
                DynChoose.from_param('Long press enabled', in_lp_enabled).extend_items(
                    DynWhen.from_condition('1').extend_items(
                        DynComObject.from_com_object(in_lp_co),
                        DynParameter.from_parameter(long_press_delay_param)
                    )
                )
            )
        )
    ))

"""
    Adding generated to project
"""


def uid_sorted(items: Iterable[dict]) -> List[dict]:
    return list(sorted(items, key=lambda item: item.get('Uid') or item.get('UId') or -1))


generated = {
    'ParameterTypes': uid_sorted([param_type.model_dump() for param_type in param_types]),
    'Parameters': uid_sorted([param.model_dump() for param in parameters]),
    'ComObjects': uid_sorted([co.model_dump() for co in com_objects]),
    'Dynamics': [dynamics.model_dump()],

    # Generate refs
    'ParameterRefs': [param.get_ref().model_dump() for param in parameters],
    'ComObjectRefs': [co.get_ref().model_dump() for co in com_objects],
}

pprint(generated)
Path('./generated.json').write_text(json.dumps(generated, indent=2))

input_path = Path('./H8I8O_V4.ae-manu')
output_path = Path('./H8I8O_gen.ae-manu')

data = json.loads(input_path.read_text())
data['Applications'][-1]['Versions'][-1]['Version'] = json.loads(
    data['Applications'][-1]['Versions'][-1]['Version']
)

data['Applications'][-1]['Versions'][-1]['Version'].update(generated)

data['Applications'][-1]['Versions'][-1]['Version'] = json.dumps(
    data['Applications'][-1]['Versions'][-1]['Version']
)
output_path.write_text(json.dumps(data))
