import re
import json
from typing import *
from pydantic import AliasChoices

with_optional = True
with_default = True
with_alias_choices = False

raw_data = """
"Name": "SEND_CYCLIC",
                    "Value": 1,
                    "UseIcon": false,
                    "IconId": -1,
                    "Translate": true,
"""

raw_data = '{' + raw_data.strip().strip(',') + '}'
data: dict = json.loads(raw_data)


def quote(s: str) -> str:
    return f'\'{s}\''


type_names_map = {
    type(None).__name__: Any.__name__,
    list.__name__: List.__name__
}

for key, value in data.items():
    python_key = '_'.join(map(lambda s: str(s).lower(), re.findall('[A-Z][^A-Z]*', key)))
    value_type_name: str = type(value).__name__
    field_data = dict()

    value_type_name = type_names_map.get(value_type_name) or value_type_name

    if with_alias_choices:
        field_names = ', '.join([quote(key), quote(python_key)])
        field_data['alias'] = f'{AliasChoices.__name__}({field_names})'
    else:
        field_data['alias'] = quote(key)
    if with_optional:
        value_type_name = f'{Optional.__name__}[{value_type_name}]'
    if with_default:
        field_data['default'] = value

    field_kwargs = ', '.join([f'{k}={v}' for k, v in field_data.items()])
    print(f'{python_key}:{value_type_name} = Field({field_kwargs})')
