from typing import *
from pydantic import *
from pydantic import BaseModel as _BaseModel


def type_field(type_name: str):
    return Field(alias='$type', default=type_name)


class BaseModel(_BaseModel):
    model_config = ConfigDict(extra='allow')
    type: Optional[str] = type_field('')

    def model_dump(self, *args, **kwargs) -> dict[str, Any]:
        data = _BaseModel.model_dump(
            self,
            exclude_unset=False,
            exclude_defaults=False,
            by_alias=True,
            warnings=False
        )
        if 'type' in data:
            data['$type'] = data.pop('type')
        return data


class DynamicsBase(BaseModel):
    pass


T = TypeVar('T', Any, DynamicsBase)


class DynamicsWithItemsBase(DynamicsBase):
    items: Optional[List[T]] = Field(alias='Items', default_factory=list)

    @staticmethod
    def _filter_items(items: Iterable[T]) -> List[T]:
        return [item for item in items if isinstance(item, DynamicsBase)]

    def extend_items(self, *items: T) -> T:
        self.items.extend(self._filter_items(items))
        return self

    def with_items(self, *items: T) -> T:
        self.items = list(self._filter_items(items))
        return self
