import json
from pathlib import Path
from models.base_model import *


class DataPointSubType(BaseModel):
    type: Optional[str] = type_field('Kaenx.Creator.Models.DataPointSubType, Kaenx.Creator')

    name: str = Field(alias='Name')
    number: str = Field(alias='Number')
    parent_number: str = Field(alias='ParentNumber')


class DataPointType(BaseModel):
    type: Optional[str] = type_field('Kaenx.Creator.Models.DataPointType, Kaenx.Creator')

    name: str = Field(alias='Name', )
    number: str = Field(alias='Number')
    size: int = Field(alias='Size')
    sub_types: List[DataPointSubType] = Field(alias='SubTypes', default_factory=list)

    @classmethod
    def get_data_points(cls) -> List['DataPointType']:
        path = Path(__file__).parent / 'datapoints.json'
        data = json.loads(path.read_text())
        return [cls.model_validate(item) for item in data]

    @classmethod
    def get_type_by_number(cls, type_number: str) -> DataPointSubType:
        return next(filter(lambda i: i.number == type_number, cls.get_data_points()))

    @classmethod
    def get_sub_type_by_number(cls, type_number: str, sub_type_number: str) -> DataPointSubType:
        return next(filter(lambda i: i.number == sub_type_number, cls.get_type_by_number(type_number).sub_types))
