from enum import IntEnum
from models.base_model import *
from models.translation import Translation


class DynSeparatorHint(IntEnum):
    NONE = 0
    RULER = 1
    HEADLINE = 2
    INFO = 3
    ERROR = 4


class DynSeparator(DynamicsWithItemsBase):
    type: Optional[str] = type_field('Kaenx.Creator.Models.Dynamic.DynSeparator, Kaenx.Creator')

    is_expanded: Optional[bool] = Field(alias='IsExpanded', default=False)
    use_text_parameter: Optional[bool] = Field(alias='UseTextParameter', default=False)
    id: Optional[int] = Field(alias='Id', default=-1)
    use_icon: Optional[bool] = Field(alias='UseIcon', default=False)
    translation_text: Optional[bool] = Field(alias='TranslationText', default=False)
    text: Optional[List[Translation]] = Field(alias='Text', default=Translation.from_string(str()))
    hint: Optional[DynSeparatorHint] = Field(alias='Hint', default=DynSeparatorHint.NONE)
    cell: Optional[Any] = Field(alias='Cell', default=None)
    access: Optional[int] = Field(alias='Access', default=2)
    name: Optional[str] = Field(alias='Name', default_factory=str)
    text_ref: Optional[int] = Field(alias='TextRef', default=-1)
    icon_id: Optional[int] = Field(alias='IconId', default=-1)
    items: Optional[Any] = Field(alias='Items', default=None)

    @classmethod
    def from_args(cls, name: str,
                  hint: Optional[DynSeparatorHint] = DynSeparatorHint.NONE,
                  text: Optional[str] = None) -> 'DynSeparator':
        return cls.model_validate({
            'Name': name,
            'Text': Translation.from_string(text or name),
            'Hint': hint
        })
