from models.base_model import *
from models.translation import Translation
from models.parameter_type import ParameterType


class ParameterRef(BaseModel):
    type: Optional[str] = type_field('Kaenx.Creator.Models.ParameterRef, Kaenx.Creator')

    uid: int = Field(alias='Uid')
    name: str = Field(alias='Name')
    parameter: int = Field(alias='Parameter')

    id: Optional[int] = Field(alias='Id', default=-1)
    display_order: Optional[int] = Field(alias='DisplayOrder', default=-1)
    is_auto_generated: Optional[bool] = Field(alias='IsAutoGenerated', default=True)
    overwrite_value: Optional[bool] = Field(alias='OverwriteValue', default=False)
    overwrite_access: Optional[bool] = Field(alias='OverwriteAccess', default=False)
    access: Optional[int] = Field(alias='Access', default=2)
    value: Optional[str] = Field(alias='Value', default_factory=str)
    translation_text: Optional[bool] = Field(alias='TranslationText', default=False)
    translation_suffix: Optional[bool] = Field(alias='TranslationSuffix', default=False)
    text: Optional[List[Translation]] = Field(alias='Text', default=Translation.from_string(str()))
    suffix: Optional[List[Translation]] = Field(alias='Suffix', default=Translation.from_string(str()))
    overwrite_text: Optional[bool] = Field(alias='OverwriteText', default=False)
    overwrite_suffix: Optional[bool] = Field(alias='OverwriteSuffix', default=False)


class Parameter(BaseModel):
    type: Optional[str] = type_field('Kaenx.Creator.Models.Parameter, Kaenx.Creator')

    uid: int = Field(alias='Uid')
    name: str = Field(alias='Name')
    parameter_type: int = Field('ParameterType')
    value: str = Field(alias='Value')

    save_path: Optional[int] = Field(alias='SavePath', default=1)
    memory_id: Optional[int] = Field(alias='MemoryId', default=1)
    id: Optional[int] = Field(alias='Id', default=-1)
    union_id: Optional[int] = Field(alias='UnionId', default=-1)
    is_in_union: Optional[bool] = Field(alias='IsInUnion', default=False)
    is_in_union_default: Optional[bool] = Field(alias='IsUnionDefault', default=False)
    offset: Optional[int] = Field(alias='Offset', default=0)
    offset_bit: Optional[int] = Field(alias='OffsetBit', default=0)
    text: Optional[List[Translation]] = Field(alias='Text', default=Translation.from_string(str()))
    suffix: Optional[List[Translation]] = Field(alias='Suffix', default=Translation.from_string(str()))
    translation_text: Optional[bool] = Field(alias='TranslationText', default=False)
    translation_suffix: Optional[bool] = Field(alias='TranslationSuffix', default=False)
    access: Optional[int] = Field(alias='Access', default=2)

    @classmethod
    def from_args(cls, uid: int, name: str, parameter_type: int | ParameterType, value: Union[str, int],
                  text: Optional[str] = None, suffix: Optional[str] = None) -> 'Parameter':
        return cls.model_validate({
            'Uid': uid,
            'Name': name,
            'ParameterType': parameter_type if type(parameter_type) is int else parameter_type.uid,
            'Value': str(value),
            'Text': Translation.from_string(text or name),
            'Suffix': Translation.from_string(suffix or str())
        }, strict=False)

    def get_ref(self) -> ParameterRef:
        ref_data = self.model_dump(exclude_unset=False, exclude_defaults=False, by_alias=True)
        ref_data.pop('$type')
        ref_data.update({'Parameter': self.uid})
        return ParameterRef.model_validate(ref_data)


if __name__ == '__main__':
    p = Parameter.from_args(uid=1, name='test param', parameter_type=1, value="0")
    print(p)
    print(p.model_dump())

    print(p.get_ref())
    print(p.get_ref().model_dump())
