from enum import IntEnum
from models.base_model import *
from models.translation import Translation


class ParameterTypeEnum(BaseModel):
    type: Optional[str] = type_field('Kaenx.Creator.Models.ParameterTypeEnum, Kaenx.Creator')

    name: str = Field(alias='Name')
    value: int = Field(alias='Value')
    text: Optional[List[Translation]] = Field(alias='Text', default=Translation.from_string(str()))

    use_icon: Optional[bool] = Field(alias='UseIcon', default=False)
    icon_id: Optional[int] = Field(alias='IconId', default=-1)
    translate: Optional[bool] = Field(alias='Translate', default=True)

    @classmethod
    def from_args(cls, name: str, value: int, text: Optional[str] = None):
        return cls.model_validate({
            'Name': name,
            'Value': value,
            'Text': Translation.from_string(text)
        })


class ParameterTypes(IntEnum):
    ENUM = 2
    NUMBER_UINT = 8


class ParameterType(BaseModel):
    type: Optional[str] = type_field('Kaenx.Creator.Models.ParameterType, Kaenx.Creator')

    uid: int = Field(alias='UId')
    name: str = Field(alias='Name')
    param_type: ParameterTypes = Field(alias='Type')
    min: str = Field(alias='Min')
    max: str = Field(alias='Max')
    increment: Optional[str] = Field(alias='Increment', default=1)
    enums: Optional[List] = Field(alias='Enums', default_factory=list)

    size_in_bit: Optional[int] = Field(alias='SizeInBit', default=-1)
    ui_hint: Optional[Literal['', 'Slider', 'CheckBox', 'ProgressBar']] = Field(alias='UIHint', default_factory=str())

    is_size_manual: Optional[bool] = Field(alias='IsSizeManual', default=False)
    translate_enums: Optional[bool] = Field(alias='TranslateEnums', default=True)
    other_value: Optional[bool] = Field(alias='OtherValue', default=True)
    baggage_u_id: Optional[int] = Field(alias='BaggageUId', default=-1)

    display_offset: Optional[str] = Field(alias='DisplayOffset', default=0)
    display_factor: Optional[str] = Field(alias='DisplayFactor', default=1)

    @classmethod
    def from_args(cls, uid: int, name: str, param_type: ParameterTypes,
                  v_min: Optional[str | int] = None, v_max: Optional[str | int] = None,
                  v_increment: Optional[str | int] = None):
        return cls.model_validate({
            'UId': uid,
            'Name': name,
            'Type': param_type,
            'Min': str(v_min or 0),
            'Max': str(v_max or 255),
            'Increment': str(v_increment or 1),
            'UIHint': ''
        })

    def with_enums(self, *enums: ParameterTypeEnum) -> 'ParameterType':
        self.enums = list(enums)
        return self

    def with_hint(self, hint: str) -> 'ParameterType':
        self.ui_hint = hint
        return self
