from models.base_model import *


class Language(BaseModel):
    type: Optional[str] = type_field('Kaenx.Creator.Models.Language, Kaenx.Creator')
    text: str = Field(alias='Text')
    culture_code: str = Field(alias='CultureCode')

    @classmethod
    def get_language_en(cls) -> 'Language':
        return cls.model_validate({
            "Text": "English (United States)",
            "CultureCode": "en-US"
        })

    @classmethod
    def get_language_de(cls) -> 'Language':
        return cls.model_validate({
            "Text": "Deutsch",
            "CultureCode": "de-DE"
        })


class Translation(BaseModel):
    type: Optional[str] = Field(alias='$type', default='Kaenx.Creator.Models.Translation, Kaenx.Creator')
    language: Language = Field(alias='Language', default_factory=Language.get_language_en)
    text: str = Field(alias='Text')
    preview: str = Field(alias='Preview')

    @classmethod
    def from_string_single(cls, s: str) -> 'Translation':
        return cls.model_validate({
            "Text": s,
            "Preview": s
        })

    @classmethod
    def from_string(cls, s: str) -> List['Translation']:
        return [cls.from_string_single(s)]


if __name__ == '__main__':
    t = Translation.from_string_single('Hello World!')
    print(t)
    print(t.model_dump())
